
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include <RP2040.h>
#include <system_RP2040.h>
#include <core_cm0plus.h>
#include <hardware/structs/iobank0.h>
#include <hardware/adc.h>
#include <hardware/clocks.h>
#include <hardware/gpio.h>
#include <hardware/irq.h>
#include <hardware/vreg.h>
#include <pico/multicore.h>
#include <pico/platform.h>
#include <pico/stdlib.h>

#include "glitch.h"

#define __STRINGIFY(f) #f
#define STRINGIFY(f) __STRINGIFY(f)
#define CORE0_FUNC(f) __scratch_x(STRINGIFY(f)) f
#define CORE1_FUNC(f) __scratch_y(STRINGIFY(f)) f

volatile struct glitch_params glitch_param_cur = {0};
#define param_cur glitch_param_cur


// PCG impl lifted from Wikipedia
#define PCG_MULTIPLIER 6364136223846793005uLL
static uint64_t mcg_state = 0xcafef00dd15ea5e5uLL; // Must be odd

static uint32_t CORE1_FUNC(pcg32_fast)(void) {
	uint64_t x = mcg_state;
	unsigned count = (unsigned)(x >> 61);	// 61 = 64 - 3

	mcg_state = x * PCG_MULTIPLIER;
	x ^= x >> 22;
	return (uint32_t)(x >> (22 + count));	// 22 = 32 - 3 - 7
}
static void CORE1_FUNC(pcg32_fast_init)(uint64_t seed) {
	mcg_state = 2*seed + 1;
	(void)pcg32_fast();
}

// ADC value is in low 12 bits of adc_read()
// FIXME: length_min_us = -1 => adc breaks? | oops was signedness bug
#define CORE1_PRE_CALC() /* BIG TODO HERE! */ \
	bool sweep = param_cur.offset_min_us == param_cur.offset_max_us \
		&& param_cur.length_min_us >= 0; \
	uint32_t len, off = param_cur.offset_min_us + \
		(pcg32_fast() % (param_cur.offset_max_us - param_cur.offset_min_us)), \
			 iom = 1u << param_cur.glitch_out_pin; \
	do { \
		if (sweep) { \
			if (len_prev == param_cur.length_max_us) len_prev = (len = param_cur.length_min_us); \
			else len = (len_prev = (len_prev + 1));\
		} else { \
			len = (param_cur.length_min_us < 0) \
				? (((uint32_t)adc_read() * param_cur.length_max_us) >> 12) \
				: (param_cur.length_min_us + (pcg32_fast() % \
					(param_cur.length_max_us - param_cur.length_min_us))); \
		} \
	} while (0) \

#define CORE1_DO_GLITCH() \
	do { \
		busy_wait_us_32(off); \
		sio_hw->gpio_set = iom; \
		busy_wait_us_32(len); \
		sio_hw->gpio_clr = iom; \
		param_cur.offset_cur = off; \
		param_cur.length_cur = len; \
	} while (0) \

static void CORE1_FUNC(glitch_core1_thread)(void) {
	// use stdlib randomness only for the seed, but use a custom rng sitting in
	// SRAM 5 to avoid bus contention
	pcg32_fast_init(random());

	int32_t len_prev = param_cur.length_min_us - 1;

	__disable_irq();

	if (param_cur.trigger_in_pin < 0) {
		SCB->SCR &= ~SCB_SCR_SEVONPEND_Msk; // don't resume WFE on interrupt

		const int irq = SIO_IRQ_PROC1;

		irq_set_enabled(irq, false);
		multicore_fifo_clear_irq();
		irq_set_priority(irq, PICO_HIGHEST_IRQ_PRIORITY);
		//delayt1_irq_enable(pio0);
		//irq_set_exclusive_handler(irq, core1_irq);
		__disable_irq();
		irq_set_enabled(irq, true);

		while (true) {
			CORE1_PRE_CALC();
			bool arm = *(volatile bool*)&param_cur.armed;

			__WFI();
			irq_set_enabled(irq, false);
			if (!arm) goto continue_;
			arm = *(volatile bool*)&param_cur.armed;
			if (!arm) goto continue_;

			CORE1_DO_GLITCH();
			multicore_fifo_drain();
			multicore_fifo_clear_irq();
		continue_:
			irq_set_enabled(irq, true);
		}

		/*while (true) {
			CORE1_PRE_CALC();
			bool arm = *(volatile bool*)&param_cur.armed;

			__WFE();
			if (!arm) continue;
			arm = *(volatile bool*)&param_cur.armed;
			if (!arm) continue;

			CORE1_DO_GLITCH();
		}*/
	} else {
		// init gpio irq
		const int gpio = param_cur.trigger_in_pin;
		const int irq = IO_IRQ_BANK0;
		const int event = GPIO_IRQ_EDGE_RISE;
		irq_set_enabled(irq, false);
		iobank0_hw->intr[gpio>>3] = event << 4*(gpio&7); // acknowledge irq
		irq_set_priority(irq, PICO_HIGHEST_IRQ_PRIORITY);
		hw_set_bits(&iobank0_hw->proc1_irq_ctrl.inte[gpio>>3], event << 4*(gpio&7));
		irq_set_enabled(irq, true);

		while (true) {
			CORE1_PRE_CALC();

			__WFI();
			iobank0_hw->intr[gpio>>3] = event << 4*(gpio&7); // acknowledge irq
			bool arm = *(volatile bool*)&param_cur.armed;
			if (!arm) continue;

			irq_set_enabled(irq, false);
			//CORE1_DO_GLITCH();
			busy_wait_us_32(off);
			if (!(sio_hw->gpio_in & (1u<<gpio))) goto cont;
			param_cur.offset_cur = off;
			param_cur.length_cur = len;
			sio_hw->gpio_set = iom;
			asm volatile(
				"1: sub %[counter], #1\n"
				"cmp %[counter], #0\n"
				"bne 1b\n"
				:[counter]"+r"(len)
				::
			);
			//busy_wait_us_32(len);
			sio_hw->gpio_clr = iom;

		cont:
			irq_set_enabled(irq, true);
		}
	}

	// TODO: PIO variant?
}

#undef CORE1_DO_GLITCH
#undef CORE1_PRE_CALC

static void CORE0_FUNC(glitch_stop_no_clock_chg)(void) {
	multicore_reset_core1();

	if (param_cur.impl != glitch_impl__none) {
		if (param_cur.impl == glitch_impl_pio) {
			// TODO: deinit PIO when implemented
		}

		// deinit GPIO
		if (param_cur.trigger_in_pin >= 0) {
			hw_write_masked(&padsbank0_hw->io[param_cur.trigger_in_pin]
				, (PADS_BANK0_GPIO0_IE_BITS)
				| (PADS_BANK0_GPIO0_SCHMITT_BITS)
				, (PADS_BANK0_GPIO0_IE_BITS | PADS_BANK0_GPIO0_OD_BITS)
				| (PADS_BANK0_GPIO0_SCHMITT_BITS)
			);
			hw_write_masked(&iobank0_hw->io[param_cur.trigger_in_pin].ctrl
				, ((uint)GPIO_FUNC_NULL << IO_BANK0_GPIO0_CTRL_FUNCSEL_LSB)
				| ((uint)GPIO_OVERRIDE_NORMAL << IO_BANK0_GPIO0_CTRL_INOVER_LSB)
				, (IO_BANK0_GPIO0_CTRL_FUNCSEL_BITS)
				| (IO_BANK0_GPIO0_CTRL_INOVER_BITS)
			);
		}

		sio_hw->gpio_clr = 1u << param_cur.glitch_out_pin;
		sio_hw->gpio_oe_clr = 1u << param_cur.glitch_out_pin;
		hw_write_masked(&padsbank0_hw->io[param_cur.glitch_out_pin]
			, (PADS_BANK0_GPIO0_IE_BITS)
			| ((uint)GPIO_SLEW_RATE_SLOW << PADS_BANK0_GPIO0_SLEWFAST_LSB)
			| (PADS_BANK0_GPIO0_DRIVE_VALUE_2MA << PADS_BANK0_GPIO0_DRIVE_LSB)
			, (PADS_BANK0_GPIO0_IE_BITS | PADS_BANK0_GPIO0_OD_BITS)
			| PADS_BANK0_GPIO0_SLEWFAST_BITS | PADS_BANK0_GPIO0_DRIVE_BITS
		);
		hw_write_masked(&iobank0_hw->io[param_cur.glitch_out_pin].ctrl
			, ((uint)GPIO_FUNC_NULL << IO_BANK0_GPIO0_CTRL_FUNCSEL_LSB)
			| ((uint)GPIO_OVERRIDE_NORMAL << IO_BANK0_GPIO0_CTRL_OUTOVER_LSB)
			, (IO_BANK0_GPIO0_CTRL_FUNCSEL_BITS)
			| (IO_BANK0_GPIO0_CTRL_OUTOVER_BITS)
		);
	}

	memset(&param_cur, 0, sizeof param_cur);
}

bool CORE0_FUNC(glitch_ready)(const struct glitch_params* params) {
	// check params values
	if (params->offset_max_us < params->offset_min_us) return false;
	if (params->length_max_us < params->length_min_us) return false;
	if (params->trigger_in_pin < -1 || params->trigger_in_pin >= 28
			|| (params->trigger_in_pin >= 23 && params->trigger_in_pin <= 25))
		return false;
	if (params->glitch_out_pin < 0 || params->glitch_out_pin >= 28
			|| (params->glitch_out_pin >= 23 && params->glitch_out_pin <= 25))
		return false;
	if (params->trigger_in_polarity != glitch_positive
			&& params->trigger_in_polarity != glitch_negative) return false;
	if (params->glitch_out_polarity != glitch_positive
			&& params->glitch_out_polarity != glitch_negative) return false;
	if (params->impl != glitch_impl_core1 && params->impl != glitch_impl_pio)
		return false;

	glitch_stop();
	memcpy(&param_cur, params, sizeof param_cur); // apply new params

	param_cur.offset_cur = 0;
	param_cur.length_cur = 0;
	param_cur.armed = false;

	// let's not care about impl and always use core1 for now
	// TODO: if pio: check SM availability etc (& clear param_cur if fails)

	// overclock to 200 MHz
	vreg_set_voltage(VREG_VOLTAGE_1_15);
	set_sys_clock_khz(200*1000, true);

	uint func = GPIO_FUNC_SIO; // TODO: PIO when implemented

	if (params->length_min_us < 0) {
		// ADC1: ADC mode
		adc_init();
		adc_gpio_init(27);
		adc_select_input(1);
		// ADC0 = GPIO26 = fixed high (as vref)
		sio_hw->gpio_set = 1u << 26;
		sio_hw->gpio_oe_set = 1u << 26;
		gpio_set_function(26, GPIO_FUNC_SIO);
	} else {
		gpio_set_function(26, GPIO_FUNC_NULL);
	}

	if (params->trigger_in_pin >= 0) {
		//gpio_set_input_hysteresis_enabled(params->trigger_in_pin, false);
		//gpio_set_dir(params->trigger_in_pin, GPIO_IN );
		//gpio_set_inover(params->trigger_in_pin,
		//	params->trigger_in_polarity ? GPIO_OVERRIDE_INVERT : GPIO_OVERRIDE_NORMAL);
		//gpio_set_function(params->trigger_in_pin, func);

		// perform all of these at once to minimize the amount of spurious edges
		sio_hw->gpio_oe_clr = 1u << param_cur.trigger_in_pin;
		hw_write_masked(&padsbank0_hw->io[param_cur.trigger_in_pin]
			, (PADS_BANK0_GPIO0_IE_BITS)
			, (PADS_BANK0_GPIO0_IE_BITS | PADS_BANK0_GPIO0_OD_BITS)
			| (PADS_BANK0_GPIO0_SCHMITT_BITS)
		);
		hw_write_masked(&iobank0_hw->io[param_cur.trigger_in_pin].ctrl
			, (func << IO_BANK0_GPIO0_CTRL_FUNCSEL_LSB)
			| ((uint)(param_cur.trigger_in_polarity ? GPIO_OVERRIDE_INVERT
				: GPIO_OVERRIDE_NORMAL) << IO_BANK0_GPIO0_CTRL_INOVER_LSB)
			, (IO_BANK0_GPIO0_CTRL_FUNCSEL_BITS)
			| (IO_BANK0_GPIO0_CTRL_INOVER_BITS)
		);
	}

	//gpio_put(params->glitch_out_pin, false);
	// TODO: gpio_set_drive_strength(PIN_PULSE, GPIO_DRIVE_STRENGTH_12MA);
	//gpio_set_slew_rate(params->glitch_out_pin, GPIO_SLEW_RATE_FAST);
	//gpio_set_dir(params->glitch_out_pin, GPIO_OUT);
	//gpio_set_outover(params->glitch_out_pin,
	//	params->glitch_out_polarity ? GPIO_OVERRIDE_INVERT : GPIO_OVERRIDE_NORMAL);
	//gpio_set_function(params->glitch_out_pin, func);

	// perform all of these at once to minimize the amount of spurious glitches
	sio_hw->gpio_clr = 1u << param_cur.glitch_out_pin;
	sio_hw->gpio_oe_set = 1u << param_cur.glitch_out_pin;
	hw_write_masked(&padsbank0_hw->io[param_cur.glitch_out_pin]
		, PADS_BANK0_GPIO0_IE_BITS
		| ((uint)GPIO_SLEW_RATE_FAST << PADS_BANK0_GPIO0_SLEWFAST_LSB)
		| (PADS_BANK0_GPIO0_DRIVE_VALUE_12MA << PADS_BANK0_GPIO0_DRIVE_LSB)
		, PADS_BANK0_GPIO0_IE_BITS | PADS_BANK0_GPIO0_OD_BITS
		| PADS_BANK0_GPIO0_SLEWFAST_BITS | PADS_BANK0_GPIO0_DRIVE_BITS
	);
	hw_write_masked(&iobank0_hw->io[param_cur.glitch_out_pin].ctrl
		, (func << IO_BANK0_GPIO0_CTRL_FUNCSEL_LSB)
		| ((uint)(param_cur.glitch_out_polarity ? GPIO_OVERRIDE_INVERT
			: GPIO_OVERRIDE_NORMAL) << IO_BANK0_GPIO0_CTRL_OUTOVER_LSB)
		, IO_BANK0_GPIO0_CTRL_FUNCSEL_BITS | IO_BANK0_GPIO0_CTRL_OUTOVER_BITS
	);

	multicore_launch_core1(glitch_core1_thread);

	return true;
}

void CORE0_FUNC(glitch_stop)(void) {
	glitch_stop_no_clock_chg();
	set_sys_clock_khz(125*1000, true);
	vreg_set_voltage(VREG_VOLTAGE_DEFAULT);
}

