
#ifndef GLITCH_H_
#define GLITCH_H_

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

enum glitch_polarity {
	glitch_positive,
	glitch_negative,
};
enum glitch_impl {
	glitch_impl__none,
	glitch_impl_core1,
	glitch_impl_pio
};

struct glitch_params {
	uint32_t offset_min_us;
	uint32_t offset_max_us;
	 int32_t length_min_us;
	 int32_t length_max_us;
	int trigger_in_pin; // use -1 for a signal coming from core 0
	int glitch_out_pin;
	enum glitch_polarity trigger_in_polarity;
	enum glitch_polarity glitch_out_polarity;
	enum glitch_impl impl;
	uint32_t offset_cur, length_cur;
	bool armed;
};

extern volatile struct glitch_params glitch_param_cur;

// return false: something wrong about the parameters
bool glitch_ready(const struct glitch_params* params);
void glitch_stop(void);

static inline void glitch_trigger_sw_core1(void) {
	//asm volatile("sev");
	sio_hw->fifo_wr = 1;
}
static inline void glitch_trigger_sw_pio(void) {
	// TODO: implement
}

static inline void glitch_arm(void) {
	glitch_param_cur.armed = true;
}
static inline void glitch_disarm(void) {
	glitch_param_cur.armed = false;
}

#endif

