
#include <stdio.h>

#include "pio_sbw.h"

#include "tap.h"

// FIXME: TDI in run-test/idle ALWAYS TCLK value??

void sbw_tap_reset(void) {
    // TDI always 1
    // TMS=1,1,1,1,1,1    -- reset TAP state to initial
    // TMS=0              -- test-logic-reset to run-test/idle
    // TMS=1,0,1,0,1      -- perform fuse check
    // TMS=1,0            -- back to run-test/idle

    //const uint16_t tms_seq = 0x1abf;//0x3f | (0<<6) | (0x15 << 7) | (0x1 << 12);
    const uint8_t tms_seq[2] = {0xbf,0x1a};
    sbw_tms_sequence(14, true, tms_seq);
}

void sbw_check_fuse(void) {
    // TDI always 1
    // TMS=01010110
    const uint8_t tms_seq = 0x6a;
    sbw_tms_sequence(8, true, &tms_seq);
}

static uint8_t bitswap(uint8_t in) {
    static const uint8_t lut[16] = {
        0x0, 0x8, 0x4, 0xc, 0x2, 0xa, 0x6, 0xe,
        0x1, 0x9, 0x5, 0xd, 0x3, 0xb, 0x7, 0xf
    };

    return (lut[in&0xf] << 4) | lut[in>>4];
}
static inline uint16_t bitswap16(uint16_t in) {
	//printf("bitswap in=%04x\n", in);
    return ((uint16_t)bitswap(in & 0xff) << 8) | bitswap(in >> 8);
}

// IR in = OK
// IR out = OK
// DR out = OK
// DR in = BAD!

uint16_t sbw_tap_shift_dr(uint16_t newdr) {
    // DR content is MSB-first instead of LSB-first (IR is the latter)
    // except the PIO already does bitswaps, so we need to do it for the IR instead
    newdr = bitswap16(newdr);

    // 100: run-test/idle -> select-dr-scan -> capture-dr -> shift-dr
    const uint8_t tms_seqa = 0x01;
    const uint8_t tms_seqb = 0x01 >> 1;
    sbw_tms_sequence(1, sbw_get_last_tclk(), &tms_seqa);
    sbw_tms_sequence(2, true, &tms_seqb);

    // 15 data bits with TMS=0
    // 1 data bit with TMS=1 (to exit1-dr)
    uint16_t res = 0;
    uint8_t newdr2 = newdr >> 15, resb = 0;
    sbw_sequence(15, false, (const uint8_t*)&newdr, (uint8_t*)&res); // FIXME: little-endian-only
    sbw_sequence( 1, true , &newdr2, &resb);
    res |= (uint16_t)resb << 15;

    // TMS=1 (to update-dr)
    // TMS=0 (to run-test/idle)
    const uint8_t tms_seq_2a = 0x01;
    const uint8_t tms_seq_2b = 0x01 >> 1;
    sbw_tms_sequence(1, true, &tms_seq_2a);
    sbw_tms_sequence(1, sbw_get_last_tclk(), &tms_seq_2b);

    return bitswap16(res);
}

uint8_t sbw_tap_shift_ir(uint8_t newir) {
    newir = /*bitswap(*/newir/*)*/;

    // 1100: run-test/idle -> select-dr-scan -> select-ir-scan -> capture-ir -> shift-ir
    const uint8_t tms_seqa = 0x03;
    const uint8_t tms_seqb = 0x03 >> 1;
    sbw_tms_sequence(1, sbw_get_last_tclk(), &tms_seqa);
    sbw_tms_sequence(3, true, &tms_seqb);

    // 7 data bits with TMS=0
    // 1 data bit with TMS=1 (to exit1-ir)
    uint8_t res = 0, resb = 0, newir2 = newir >> 7;
    sbw_sequence(7, false, &newir , &res );
    sbw_sequence(1, true , &newir2, &resb);
    res |= resb << 7;

    // TMS=1 (to update-ir)
    // TMS=0 (to run-test/idle)
    const uint8_t tms_seq_2a = 0x01;
    const uint8_t tms_seq_2b = 0x01 >> 1;
    sbw_tms_sequence(1, true, &tms_seq_2a);
    sbw_tms_sequence(1, sbw_get_last_tclk(), &tms_seq_2b);

    return bitswap(res);
}

