
#include <hardware/clocks.h>
#include <hardware/gpio.h>
#include <hardware/pio.h>
#include <hardware/timer.h>

#include "swim_hw.h"

#include "swim.pio.h"

#define PINOUT_SWIM_PIO pio0
#define PINOUT_SWIM     22

int swim_tx_sm = -1, swim_rx_sm = -1, swim_tx_off = -1, swim_rx_off = -1;

void swim_preinit(void) {
	// TODO:
	// * init GPIO (pulls, dir, out, function)
	// * do entry sequence (~8 phases?)
}

bool swim_init(void) {
	// TODO:
	// for each rx, tx:
	//   * allocate PIO program space
	//   * allocate PIO SMs
	//   * init PIO SM program & stuff
	return false;
}

void swim_deinit(void) {
	// TODO:
	// * disable PIO SMs
	// * deallocate PIO SMs
	// * deallocate PIO program space
}

void swim_set_speed(float swim_clock_freq, bool fast) {
	swim_tx_set_speed(PINOUT_SWIM_PIO, swim_tx_sm, swim_tx_off, true,
			swim_clock_freq, fast);
	swim_tx_set_speed(PINOUT_SWIM_PIO, swim_rx_sm, swim_rx_off, true,
			swim_clock_freq, fast);
}

// SWIM only sends up to 10 bits at once before changing direction (for an ack)
void swim_tx_bits(uint32_t nbits, uint16_t bits) {
	// TODO:
	// * init PIO SM stuff
	// * send length
	// * send data
	// * wait for tx fifo empty
	// * wait until finished? (use IRQ?? prolly best sync mechanism)
}
uint16_t swim_rx_bits(uint32_t nbits) {
	uint16_t r = 0;

	// TODO:
	// * init PIO SM stuff
	// * send length
	// * read data (incl. wait until finished)

	return r;
}

static uint8_t bitswap(uint8_t in) {
	static const uint8_t lut[16] = {
		0x0, 0x8, 0x4, 0xc, 0x2, 0xa, 0x6, 0xe,
		0x1, 0x9, 0x5, 0xd, 0x3, 0xb, 0x7, 0xf
	};

	return (lut[in&0xf] << 4) | lut[in>>4];
}

bool swim_tx_command(uint8_t cmd, size_t datasize, const uint8_t* data) {
	// cmd is 0 bit, 3 data bits, parity bit, responded by an ack bit
	uint16_t r = 0, v = (0<<15) | ((cmd&1)<<14) | ((cmd&2)<<12) | ((cmd&4)<<10);
	// add parity bit
	v |= (((cmd>>0)^(cmd>>1)^(cmd>>2)) & 1) << 11;

	for (size_t i = 0; i < 10; ++i) {
		swim_tx_bits(5, v);

		r = swim_rx_bits(1); // ack bit
		if (r) break;
	}

	if (!r) {
		// oops, no ack!
		return false;
	}

	// start sending data bits
	for (size_t i = 0; i < datasize; ++i) {
		// calculate parity
		for (size_t j = 0; j < 8; ++j) v ^= data[i] >> j;
		v = (v & 1) << 6;
		// and value bits
		v |= (0 << 15) | ((uint16_t)bitswap(data[i]) << 7);

		for (size_t j = 0; j < 10; ++j) {
			swim_tx_bits(10, v);

			r = swim_rx_bits(1); // ack bit
			if (r) break;
		}

		if (!r) {
			// oops, no ack!
			return false;
		}
	}

	return true;
}

bool swim_rx_data(size_t datasize, uint8_t* data) {
	uint16_t v = 0, r = 0;

	for (size_t i = 0; i < datasize; ++i) {
		for (size_t j = 0; j < 10; ++j) {
			v = swim_rx_bits(10);

			if (!(v & 0x8000)) { // should be a device->host cmd
				swim_tx_bits(1, 0); // send NAK
				continue;
			}

			// calculate parity
			r = 0;
			for (size_t k = 7; k < 15; ++k) r ^= v >> k;

			if (((v >> 6) & 1) != (r & 1)) {
				// parity mismatch!
				r = 0;
				swim_tx_bits(1, 0); // send NAK
				continue;
			}

			// send ACK
			swim_tx_bits(1, 0x8000);

			data[i] = bitswap(v >> 7);

			r = 1;
			break;
		}

		if (!r) {
			// oops, something went wrong
			return false;
		}
	}

	return true;
}

