
#ifndef SWIM_HW_H_
#define SWIM_HW_H_

#include <stdint.h>
#include <stdbool.h>

// perform entry sequence
void swim_preinit(void);

bool swim_init(void);
void swim_deinit(void);

// 'swim_clock_freq' is the frequency of the underlying SWIM clock,
// *NOT* the baudrate! in the STM8 device being debugged, this is usually
// HSI/2 = 8 MHz
void swim_set_speed(float swim_clock_freq, bool fast);

// SWIM only sends up to 10 bits at once before changing direction (for an ack)
// bits are transferred most-significant-bit first (for both functions here)
void swim_tx_bits(uint32_t nbits, uint16_t bits);
uint16_t swim_rx_bits(uint32_t nbits);

bool swim_tx_command(uint8_t cmd, size_t datasize, const uint8_t* data);
bool swim_rx_data(size_t datasize, uint8_t* data);

#endif

