
#include <stdio.h>

#include <hardware/clocks.h>
#include <hardware/pio.h>
#include <hardware/pio_instructions.h>
#include <hardware/sync.h>
#include <hardware/timer.h>

#include "pio_sbw.h"
#include "sbw.pio.h"

static uint16_t pio_read_one(PIO pio, uint sm, uint32_t off, int _) {
	(void)_;

	const int FREQ_SM = 500 * 1000; // 500 kHz
	const int T_SM_US = 2; // 1/500 kHz = 2 us

	// disable SM
	pio_sm_set_enabled(pio, sm, false);
	// disable wrap
	pio_sm_config defcfg = pio_get_default_sm_config();
	pio_sm_set_config(pio, sm, &defcfg);
	// set clkdiv to something slow
	//pio_sm_set_clkdiv(pio, sm, 2*(float)clock_get_hz(clk_sys) / FREQ_SM);
	pio_sm_set_clkdiv_int_frac(pio, sm, 0, 0);
	// clear PIO FIFOs
	pio_sm_clear_fifos(pio, sm);
	// clear debug flags
	const uint32_t flags = (1<<PIO_FDEBUG_TXOVER_LSB)
	                     | (1<<PIO_FDEBUG_RXUNDER_LSB)
	                     | (1<<PIO_FDEBUG_TXSTALL_LSB)
	                     | (1<<PIO_FDEBUG_RXSTALL_LSB);
	pio->fdebug = flags;
	// restart SM
	pio_sm_restart(pio, sm);
	// restart SM clkdiv
	pio_sm_clkdiv_restart(pio, sm);
	// exec jmp to offset
	uint16_t jmp = pio_encode_jmp(off);
	pio_sm_exec(pio, sm, jmp);

	uint32_t irq = save_and_disable_interrupts();

	// enable
	pio_sm_set_enabled(pio, sm, true);
	// wait 1 cycle
	//busy_wait_us_32(T_SM_US);
	pio_sm_clkdiv_restart(pio, sm);
	// read SM insn
	uint16_t ret = pio->sm[sm].instr;
	// disable SM
	pio_sm_set_enabled(pio, sm, false);

	restore_interrupts(irq);

	return ret;
}

struct bin {
	uint16_t value;
	uint16_t noccur;
};
#define BINS 32 /* idk */
/*static uint16_t pio_get_one(PIO pio, uint sm, uint32_t off, int runs) {
	// there's some variability in the return value of pio_read_one, sometimes
	// returning the next instruction. I don't know the exact reason, but a
	// possible cause could be that the timing of the PIO instructions doesn't
	// line up properly wrt the code running on the Cortex-M0+. This could be
	// caused by a too-rounded-off clock divider for the PIO SM, the C code
	// being compiled into assembly that has too much cycle jitter, USB frame
	// interrupts messing with the timing, and maybe a few other reasons.

	static struct bin bins[BINS];
	for (int i = 0; i < BINS; ++i) {
		bins[i].value = 0;
		bins[i].noccur = 0;
	}

	for (int j = 0; j < runs; ++j) {
		uint16_t v = pio_read_one(pio, sm, off, 0);
		for (int i = 0; i < BINS; ++i) {
			if ((bins[i].value == v && bins[i].noccur > 0) || bins[i].noccur == 0) {
				bins[i].value = v;
				++bins[i].noccur;
			}
		}
	}

	// find max
	int maxind = -1, maxoccur = -1;
	for (int i = 0; i < BINS; ++i) {
		if (bins[i].noccur > maxoccur) {
			maxind = i;
			maxoccur = bins[i].noccur;
		}
	}

	return bins[maxind].value;
}*/

extern int sbw_piosm;
void piodump_main(void) {
	/*for (uint32_t i = 0; i < 32; ++i) {
		printf("program[0x%02lx] = 0x%04x\n", i, sbw_program_instructions[i]);
	}*/

	sbw_init();

	uint16_t stuff[32];
	for (uint32_t i = 0; i < 32; ++i) {
		stuff[i] = /*pio_get_one*/pio_read_one(PINOUT_SBW_PIO, (uint)sbw_piosm, i, 8);
		bool good = stuff[i] == sbw_program_instructions[i];
		printf("PIO_INSN[0x%02lx] = 0x%04x (%s)\n", i, stuff[i],
				good ? "correct" : "wrong");
	}

	sbw_deinit();
}

