
#include <hardware/gpio.h>
#include <hardware/pio.h>
#include <pico/time.h>

#include "tool78_defs.h"
#include "tool78_hw_helpers.h"

#include "tool78.pio.h"

#include "tool78_hw.h"

static struct tool78_pio_vars vars;

static int t78k0_uart2_send(int len, const uint8_t* data, int32_t timeout_us);

static bool t78k0_uart2_init(void) {
	if (!tool78_hw_init_help(&tool78_uart_tx_program, &tool78_uart_rx_program,
				&vars)) return false;

	if (tool78_hw_rl78_uart2.flags & tool78_hw_flag_do_ocd)
		return false; // OCD not supported with this phy
	vars.exclusive = false;
	vars.bitswap = true;

	// extclk between 2 and 20 MHz
	tool78_entryseq_78k0(tool78_entry_78k0_uart2);

	tool78_uart_tx_program_init(PINOUT_TOOL78_PIO, vars.smtx, vars.txoff,
			PINOUT_TOOL78_78K0_TXMOSI, 9600, true);
	tool78_uart_rx_program_init(PINOUT_TOOL78_PIO, vars.smrx, vars.rxoff,
			PINOUT_TOOL78_78K0_RXMISO, 9600, true);

	// magic initial not-really-commands
	// wait tCOM or tR1 (unclear) (>445k/8M s + (2^16)/2M s)
	// send 0x00
	// wait t12 (>15k/8M s)
	// send 0x00
	// wait t2C (>15k/8M s)

	busy_wait_us_32(477768); // tCOM or tR1

	uint8_t byte = 0x00;
	t78k0_uart2_send(1, &byte, -1);

	busy_wait_us_32(1900); // t12

	t78k0_uart2_send(1, &byte, -1);

	busy_wait_us_32(1900); // t2C

	// now a reset command needs to be sent, but we leave that to the upper
	// (command processing) layer as it has to know about those timings anyway

	return true; // all is well!
}
static void t78k0_uart2_deinit(void) {
	tool78_hw_deinit_help(&tool78_uart_tx_program, &tool78_uart_rx_program,
			&vars);

	tool78_deinit_78k0();
}

static int t78k0_uart2_has_available(void) {
	return tool78_hw_has_available_help(&vars);
}

static int t78k0_uart2_recv(int len, uint8_t* data, int32_t timeout_us) {
	return tool78_hw_help_recv(&vars, len, data, timeout_us);
}
static int t78k0_uart2_send(int len, const uint8_t* data, int32_t timeout_us) {
	return tool78_hw_help_send(&vars, 12/*inter-byte delay (us) tDR = 90/8M s */,
			len, data, timeout_us);
}

static void t78k0_uart2_set_baudrate(uint32_t baudrate) {
	float div = (float)clock_get_hz(clk_sys) / (8*baudrate);
	pio_sm_set_clkdiv(PINOUT_TOOL78_PIO, vars.smrx, div);
	pio_sm_set_clkdiv(PINOUT_TOOL78_PIO, vars.smtx, div);
}

/*static void t78k0_uart2_rx_set_stop_bit(bool enable) {
	tool78_rx_set_stop_bit(&vars, enable);
}*/

struct tool78_hw tool78_hw_78k0_uart2 = {
	.target = tool78k0_uart2,

	.init = t78k0_uart2_init,
	.deinit = t78k0_uart2_deinit,

	.set_baudrate = t78k0_uart2_set_baudrate,
	.has_available = t78k0_uart2_has_available,

	.recv = t78k0_uart2_recv,
	.send = t78k0_uart2_send,

	//.rx_set_stop_bit = t78k0_uart2_rx_set_stop_bit
};

