
#include <hardware/dma.h>
#include <hardware/gpio.h>
#include <hardware/irq.h>
#include <hardware/pio.h>
#include <pico/time.h>
#include <pico/timeout_helper.h>

#include "tool78_defs.h"
#include "tool78_hw_helpers.h"

#include "tool78.pio.h"

#include "tool78_hw.h"

static struct tool78_pio_vars vars;

static bool test_init(void) {
	if (!tool78_hw_init_help(&tool78_uart_tx_program, &tool78_uart_rx_program,
				&vars)) return false;

	vars.exclusive = false;
	vars.bitswap = true;

	// TODO: entry sequence here

	tool78_uart_tx_program_init(PINOUT_TOOL78_PIO, vars.smtx, vars.txoff,
			PINOUT_TOOL78_RL78_TX, 115200, true);
	tool78_uart_rx_program_init(PINOUT_TOOL78_PIO, vars.smrx, vars.rxoff,
			PINOUT_TOOL78_RL78_RX, 115200, true);

	// TODO: magic initial not-really-commands

	return true; // all is well!
}
static void test_deinit(void) {
	tool78_hw_deinit_help(&tool78_uart_tx_program, &tool78_uart_rx_program,
			&vars);

	// keep pullups
	gpio_set_function(PINOUT_TOOL78_78K0_TXMOSI, GPIO_FUNC_NULL);
	gpio_set_function(PINOUT_TOOL78_78K0_RXMISO, GPIO_FUNC_NULL);
}

static int test_has_available(void) {
	return tool78_hw_has_available_help(&vars);
}

static int test_recv(int len, uint8_t* data, int32_t timeout_us) {
	return tool78_hw_help_recv(&vars, len, data, timeout_us);
}
static int test_send(int len, const uint8_t* data, int32_t timeout_us) {
	return tool78_hw_help_send(&vars, 0/*inter-byte delay*/,
			len, data, timeout_us);
}

static void test_set_baudrate(uint32_t baudrate) {
	float div = (float)clock_get_hz(clk_sys) / (8*baudrate);
	pio_sm_set_clkdiv(PINOUT_TOOL78_PIO, vars.smrx, div);
	pio_sm_set_clkdiv(PINOUT_TOOL78_PIO, vars.smtx, div);
}

/*static void test_uart2_rx_set_stop_bit(bool enable) {
	tool78_rx_set_stop_bit(&vars, enable);
}*/

struct tool78_hw tool78_hw_test_uart2 = {
	.target = tool78k0_uart2,

	.init = test_init,
	.deinit = test_deinit,

	.set_baudrate = test_set_baudrate,
	.has_available = test_has_available,

	.recv = test_recv,
	.send = test_send,

	//.rx_set_stop_bit = test_uart2_rx_set_stop_bit
};

