
#include <stdint.h>
#include <stdbool.h>

#include <hardware/structs/iobank0.h>
#include <hardware/gpio.h>
#include <hardware/spi.h>

#include "zap_pinout.h"

#include "zap_dac.h"

static enum zap_dac_ch gain, enable;
static uint16_t va, vb;

void zap_dac_init(bool shdn) {
	spi_init(ZAP_DAC_SPI, 1*1000*1000); // TODO: can we go higher?
	spi_set_format(ZAP_DAC_SPI, 16, SPI_CPOL_0, SPI_CPHA_0, SPI_MSB_FIRST);

	gpio_put(ZAP_DAC_NLATCH, true);
	gpio_put(ZAP_DAC_CS    , true);
	//gpio_put(ZAP_DAC_NSHDN , !shdn);
	gpio_set_dir(ZAP_DAC_NLATCH, GPIO_OUT);
	gpio_set_dir(ZAP_DAC_CS    , GPIO_OUT);
	//gpio_set_dir(ZAP_DAC_NSHDN , GPIO_OUT);

	gpio_set_function(ZAP_DAC_SCLK  , GPIO_FUNC_SPI);
	gpio_set_function(ZAP_DAC_TX    , GPIO_FUNC_SPI);
	gpio_set_function(ZAP_DAC_NLATCH, GPIO_FUNC_SIO);
	gpio_set_function(ZAP_DAC_CS    , GPIO_FUNC_SIO);
	//gpio_set_function(ZAP_DAC_NSHDN , GPIO_FUNC_SIO);

	enable = 0;
	gain = 3; // 1x for both
	va = 0x0fff;
	vb = 0x0800;
}

static void zap_flush(enum zap_dac_ch ch) {
	// buffer is always enabled (in MCP49x2 case, doesn't exist in MCP48x2):
	// limited BW isn't really an issue (we use the MAX4619 for that anyway),
	// and output swing is also limited by the output buffer which has the same
	// limitation. so, shrug

	if (ch & zap_dac_a) {
		uint16_t v = va;
		v |= 0x8000*0; // channel A
		v |= 0x4000; // buffered (MCP49x2, don't care on 48x2)
		if (gain & zap_dac_a) v |= 0x2000;
		if (enable & zap_dac_a) v |= 0x1000;

		sio_hw->gpio_clr = 1u << ZAP_DAC_CS;
		busy_wait_us_32(10); // TODO: better timing
		spi_write16_blocking(ZAP_DAC_SPI, &v, 1);
		busy_wait_us_32(10);
		sio_hw->gpio_set = (1u << ZAP_DAC_CS);
	}
	if (ch & zap_dac_b) {
		uint16_t v = vb;
		v |= 0x8000*1; // channel B
		v |= 0x4000; // buffered (MCP49x2, don't care on 48x2)
		if (gain & zap_dac_b) v |= 0x2000;
		if (enable & zap_dac_b) v |= 0x1000;

		sio_hw->gpio_clr = 1u << ZAP_DAC_CS;
		busy_wait_us_32(10); // TODO: better timing
		spi_write16_blocking(ZAP_DAC_SPI, &v, 1);
		busy_wait_us_32(10);
		sio_hw->gpio_set = (1u << ZAP_DAC_CS);
	}
}

enum zap_dac_ch zap_dac_get_enable(void) { return enable; }
enum zap_dac_ch zap_dac_get_mulx2(void) { return ~gain & 3; }

void zap_dac_set_enable(enum zap_dac_ch chen) {
	enum zap_dac_ch change = chen ^ enable;

	enable = chen;

	zap_flush(change);
}
void zap_dac_set_mulx2(enum zap_dac_ch ch_x2) { gain = ~ch_x2 & 3; }

uint16_t zap_dac_get_a(void) { return va; }
uint16_t zap_dac_get_b(void) { return vb; }

void zap_dac_set_a(uint16_t a) {
	va = a & 0x0fff;
	zap_flush(zap_dac_a);
}
void zap_dac_set_b(uint16_t b) {
	vb = b & 0x0fff;
	zap_flush(zap_dac_b);
}

void zap_dac_latch(void) {
	busy_wait_us_32(10); // TODO: better timing
	sio_hw->gpio_clr = 1u << ZAP_DAC_NLATCH;
	busy_wait_us_32(10);
	sio_hw->gpio_set = 1u << ZAP_DAC_NLATCH;
}

