
// code partly from https://github.com/newaetech/chipshouter-picoemp/tree/main/firmware/c

#include <stdint.h>
#include <stdbool.h>

#include <hardware/structs/iobank0.h>
#include <hardware/clocks.h>
#include <hardware/gpio.h>
#include <hardware/pwm.h>
#include <pico/time.h>

#include "zap_pinout.h"

#include "zap_picoemp.h"

static bool armed;

void zap_picoemp_init(void) {
	gpio_put(ZAP_PICOEMP_HVP, false);
	gpio_set_dir(ZAP_PICOEMP_HVP, GPIO_OUT);
	gpio_set_pulls(ZAP_PICOEMP_HVP, false, false);
	gpio_set_drive_strength(ZAP_PICOEMP_HVP, GPIO_DRIVE_STRENGTH_12MA);
	gpio_set_slew_rate(ZAP_PICOEMP_HVP, GPIO_SLEW_RATE_FAST);
	gpio_set_function(ZAP_PICOEMP_HVP, GPIO_FUNC_SIO);

	gpio_set_dir(ZAP_PICOEMP_CHG, GPIO_IN);
	gpio_set_pulls(ZAP_PICOEMP_CHG, false, false);
	gpio_set_inover(ZAP_PICOEMP_CHG, GPIO_OVERRIDE_INVERT);
	gpio_set_function(ZAP_PICOEMP_CHG, GPIO_FUNC_SIO);

	zap_picoemp_set_armed(false);
}

bool zap_picoemp_get_armed(void) { return armed; }

static uint32_t pwm_set_freq_duty(uint slice_num, uint chan, uint32_t f, float d) {
	uint32_t clock = clock_get_hz(clk_sys);
	uint32_t div16 = clock / f / 4096 + (clock % (f * 4096) != 0);

	if (div16 / 16 == 0) div16 = 16;

	uint32_t wrap = clock * 16 / div16 / f - 1;

	pwm_set_clkdiv_int_frac(slice_num, div16/16, div16 & 0xf);
	pwm_set_wrap(slice_num, wrap);
	pwm_set_chan_level(slice_num, chan, (int)((float)wrap * d));

	return wrap;
}

void zap_picoemp_set_armed(bool arm) {
	if (arm) {
		if (armed) return;

		uint32_t slice = pwm_gpio_to_slice_num(ZAP_PICOEMP_PWM);
		uint32_t chan = pwm_gpio_to_channel(ZAP_PICOEMP_PWM);

		float tar_freq = 25;
		float div = clock_get_hz(clk_sys) / tar_freq;

		pwm_config cfg = pwm_get_default_config();
		pwm_config_set_clkdiv(&cfg, div);
		pwm_config_set_wrap(&cfg, UINT16_MAX);

		pwm_init(slice, &cfg, false);
		pwm_set_chan_level(slice, chan, 800);
		pwm_set_freq_duty(slice, chan, 2500, 0.0122);

		pwm_set_enabled(slice, true);
		gpio_set_function(ZAP_PICOEMP_PWM, GPIO_FUNC_PWM);

		armed = arm;
	} else {
		armed = false;

		gpio_set_dir(ZAP_PICOEMP_PWM, GPIO_OUT);
		gpio_put(ZAP_PICOEMP_PWM, false);
		gpio_set_function(ZAP_PICOEMP_PWM, GPIO_FUNC_SIO);
	}
}

void zap_picoemp_pulse(void) {
	gpio_put(ZAP_PICOEMP_HVP, true);
	busy_wait_us_32(5);
	gpio_put(ZAP_PICOEMP_HVP, false);
	busy_wait_us_32(5);
	gpio_put(ZAP_PICOEMP_HVP, true);
	busy_wait_us_32(5);
	gpio_put(ZAP_PICOEMP_HVP, false);
	busy_wait_ms(250);
}

